@@ -3,7 +3,7 @@
  *
  * Copyright (C) 2005-2012  NTT DATA CORPORATION
  *
- * Version: 0.2.9   2021/04/01
+ * Version: 0.2.10   2021/06/06
  */
 
 #include "caitsith.h"
@@ -457,6 +457,10 @@ oom:
  * @clear: True to cleanup @r before return, false otherwise.
  *
  * Returns 0 on success, negative value otherwise.
+ *
+ * If "transition=" part was specified to "allow" entries of non "execute" acl
+ * but transition to that domain failed due to e.g. memory quota, the current
+ * thread will be killed by SIGKILL.
  */
 int cs_check_acl(struct cs_request_info *r, const bool clear)
 {
@@ -464,6 +468,17 @@ int cs_check_acl(struct cs_request_info *r, const bool clear)
 	const int idx = cs_read_lock();
 
 	error = cs_check_acl_list(r);
+	if (r->transition && r->transition != &cs_null_name &&
+	    r->result == CS_MATCHING_ALLOWED && r->type != CS_MAC_EXECUTE &&
+	    !cs_transit_domain(r->transition->name)) {
+		printk(KERN_WARNING "ERROR: Unable to transit to '%s' domain.\n",
+		       r->transition->name);
+#if LINUX_VERSION_CODE < KERNEL_VERSION(5, 3, 0)
+		force_sig(SIGKILL, current);
+#else
+		force_sig(SIGKILL);
+#endif
+	}
 	cs_read_unlock(idx);
 	if (clear)
 		cs_clear_request_info(r);
@@ -2976,22 +2991,9 @@ static void cs_check_auto_domain_transition(void)
 {
 #ifdef CONFIG_CAITSITH_AUTO_DOMAIN_TRANSITION
 	struct cs_request_info r = { };
-	const int idx = cs_read_lock();
 
 	r.type = CS_MAC_AUTO_DOMAIN_TRANSITION;
 	cs_check_acl(&r, true);
-	if (r.result != CS_MATCHING_ALLOWED ||
-	    cs_transit_domain(r.transition->name))
-		goto done;
-	printk(KERN_WARNING "ERROR: Unable to transit to '%s' domain.\n",
-	       r.transition->name);
-#if LINUX_VERSION_CODE < KERNEL_VERSION(5, 3, 0)
-	force_sig(SIGKILL, current);
-#else
-	force_sig(SIGKILL);
-#endif
-done:
-	cs_read_unlock(idx);
 #endif
 }
 
